import { type Ref, ref } from 'vue'
// import type { EChartsOption } from 'echarts/types/dist/shared'
import type { EChartsOption } from 'echarts/types/dist/shared'
import axiosInstance from '@/utils/axios/axiosInstance.ts'
import type { ResponseInfo } from '@/types/res.ts'
import type { QueryInfo } from '@/types/table.ts'
import { generateUniqueColors } from '@/utils/common'
import PieBorderRadius from '@/components/echarts/PieBorderRadius.vue'

type MChartType = InstanceType<typeof PieBorderRadius>

interface UserBehaviorChartItem {
  name: string
  value: number
}

interface ChartInfo {
  PieChart: UserBehaviorChartItem[]
  BarChart: {
    labelList: string[]
    valueList: number[]
  }
}

export function useUserBehaviorChart(
  url: string,
  queryFormData: Ref<any>,
  filterInfo: Array<QueryInfo>,
  chartNeedFields: Array<string>,
  isPie: Ref<boolean>,
  chartInstance: Ref<MChartType | null>,
  isLog: Ref<boolean>,
  chartsOptions: Ref<EChartsOption | null>
) {
  const chartInfo = ref<ChartInfo>({
    PieChart: [],
    BarChart: {
      labelList: [],
      valueList: []
    }
  })

  // 生成接近正方形的行列数
  function computeGrid(n: number) {
    const cols = Math.ceil(Math.sqrt(n))
    const rows = Math.ceil(n / cols)
    return { rows, cols }
  }

  // 根据行列数和索引计算中心点，返回百分比数字
  function generateCenters(n: number): { x: number; y: number }[] {
    const { rows, cols } = computeGrid(n)
    const centers: { x: number; y: number }[] = []
    for (let i = 0; i < n; i++) {
      const row = Math.floor(i / cols)
      const col = i % cols
      const x = ((col + 0.5) / cols) * 80
      const y = ((row + 0.5) / rows) * 100
      centers.push({ x, y })
    }
    return centers
  }

  const transformData = (data: { name: string; value: number }[]) => {
    const result: {
      [key: string]: {
        [key: string]: number
      }
    } = {}

    data.forEach(({ name, value }) => {
      const [timePart, adPart] = name.split('&&')
      const timeKey = timePart.trim()

      const adKey = adPart.trim()

      if (!result[timeKey]) {
        result[timeKey] = {}
      }

      result[timeKey][adKey] = value
    })

    return result
  }

  const updateChartData = async () => {
    try {
      if (chartNeedFields.length > 0) {
        const noFilter = chartNeedFields.every((item) => {
          return queryFormData.value[item] === ''
        })
        if (noFilter) {
          const tip = filterInfo
            .filter((item) => {
              return chartNeedFields.includes(item.name as any)
            })
            .map((item) => {
              return item.label
            })
            .join(',')
          ElMessage.warning(`${tip}请至少输入一个筛选条件`)
          return
        }
      }

      if (chartInstance.value) {
        chartInstance.value.startLoading()
      }

      const params = {} as any
      Object.assign(params, queryFormData.value)
      if (queryFormData.value.createTime) {
        params.createTime = queryFormData.value.createTime
          .map((item: Date) => new Date(item).getTime() / 1000)

          .join(',')
      }

      const res = (await axiosInstance.post(url, params)) as ResponseInfo
      if (res.code !== 0) {
        ElMessage.error('获取数据失败')
        return
      }

      const data = res.data as Array<{
        name: string
        count: number
      }> | null

      if (!data) {
        chartInfo.value.PieChart = []
        chartInfo.value.BarChart = {
          labelList: [],
          valueList: []
        }
      } else {
        const isAllZero = data.every((item) => item.count === 0)
        if (isAllZero) {
          chartInfo.value.PieChart = []
          chartInfo.value.BarChart = {
            labelList: [],
            valueList: []
          }
        } else {
          chartInfo.value.PieChart = data.map((item) => ({
            name: item.name,
            value: item.count
          }))
          chartInfo.value.BarChart = {
            labelList: data.map((item) => item.name),
            valueList: data.map((item) => item.count)
          }
        }
      }
      updateOptions()
    } catch (err) {
      console.log(err)
    } finally {
      if (chartInstance.value) {
        chartInstance.value.stopLoading()
      }
    }
  }

  const createPieChartOptions = (): EChartsOption => {
    const pieData = chartInfo.value.PieChart

    // 判断是不是只有一个筛选条件
    let hasCount = 0
    for (const [k, v] of Object.entries(queryFormData.value)) {
      if (chartNeedFields.includes(k) && v) {
        hasCount++
      }
    }
    if (hasCount === 1) {
      return {
        color: generateUniqueColors(pieData.length), // 根据条目数生成配色
        tooltip: {
          trigger: 'item',
          formatter: '{b}: {c} ({d}%)'
        },
        legend: {
          orient: 'vertical',
          left: 10,
          top: 'center',
          data: pieData.map((item) => item.name)
        },
        series: [
          {
            name: '在线时长分布',
            type: 'pie',
            radius: ['40%', '70%'], // 内外半径，也可以用 '50%' 单值
            center: ['50%', '50%'], // 居中
            avoidLabelOverlap: false,
            itemStyle: {
              borderRadius: 6,
              borderColor: '#fff',
              borderWidth: 2
            },
            label: {
              show: true,
              position: 'outside',
              formatter: '{b}: {c} ({d}%)'
            },
            emphasis: {
              label: {
                show: true,
                fontSize: 18,
                fontWeight: 'bold'
              }
            },
            data: pieData.map(({ name, value }) => ({
              name,
              value
            }))
          }
        ]
      }
    }
    // if(queryFormData.value.)
    const tsData = transformData(pieData)

    const timeKeys = Object.keys(tsData)
    const count = timeKeys.length
    const centers = generateCenters(count)

    const colors = generateUniqueColors(count)
    // 1. 生成 series
    const series = timeKeys.map((timeKey, idx) => ({
      name: timeKey,
      type: 'pie',
      radius: '30%', // 每个饼图半径
      center: [`${centers[idx].x}%`, `${centers[idx].y}%`],
      data: Object.entries(tsData[timeKey]).map(([adKey, val]) => ({
        name: adKey.split(':')[1],
        value: val
      })),
      label: { formatter: '{b}: {c} ({d}%)' }
    }))
    const titles = timeKeys.map((timeKey, idx) => {
      const { x, y } = centers[idx]

      return {
        text: timeKey,
        left: `${x}%`,
        top: `${y - 3}%`,
        textAlign: 'center'
      }
    })

    return {
      color: colors,
      title: titles,
      tooltip: {
        trigger: 'item',
        formatter: '{a} <br/>{b}: {c} ({d})'
      },
      legend: {
        type: 'scroll',
        orient: 'vertical',
        right: 10,
        top: 20,
        bottom: 20
      },
      series: series
    } as EChartsOption
  }

  const createBarChartsOptions = (): EChartsOption => {
    const barChartInfo = chartInfo.value.BarChart
    const yType: any = isLog.value ? 'log' : 'value'
    return {
      xAxis: {
        type: 'category',
        data: barChartInfo.labelList
        // axisLabel: {
        //   interval: 30,
        //   // rotate: 30,
        //   width: 10, // 增加标签宽度
        //   overflow: 'truncate' // 过长时显示省略号
        // }
      },
      yAxis: {
        type: yType,
        min: 1,
        axisLabel: {
          formatter: (value: number) => value.toLocaleString()
        }
      },
      dataZoom: [
        {
          type: 'slider',
          show: true,
          start: 0,
          end: 100,
          handleSize: 8
        },
        {
          type: 'inside',
          start: 0,
          end: 100
        },
        {
          type: 'slider',
          show: false,
          yAxisIndex: 0,
          filterMode: 'empty',
          width: 12,
          height: '70%',
          handleSize: 8,
          showDataShadow: false,
          left: '93%'
        }
      ],
      color: '#9FE080',
      tooltip: {
        trigger: 'axis',
        axisPointer: {
          type: 'shadow'
        },
        formatter: (params: any) => {
          const data = chartInfo.value.PieChart[params[0].dataIndex]
          return `${data.name}<br/>
                  数量: ${data.value}`
        }
      },
      grid: {
        top: '15%',
        left: '3%',
        right: '4%',
        bottom: '15%',
        containLabel: true
      },
      series: [
        {
          name: '用户行为',
          data: barChartInfo.valueList,
          type: 'bar',
          barWidth: '10%',
          label: {
            show: true,
            position: 'top',
            formatter: '{c}'
          }
        }
      ]
    }
  }

  const updateOptions = () => {
    const pieChartInfo = chartInfo.value.PieChart.length ?? null
    const barChartInfo = chartInfo.value.BarChart.valueList.length ?? null

    if (!pieChartInfo && !barChartInfo) {
      chartsOptions.value = null
      return
    }
    const pieChartOptions = createPieChartOptions()
    const barChartOptions = createBarChartsOptions()

    chartsOptions.value = isPie.value ? pieChartOptions : barChartOptions
  }

  return {
    updateChartData,
    updateOptions
  }
}
